#!/usr/bin/perl -I/usr/sausalito/perl -I/usr/sausalito/handlers/base/disk
# $Id: Disk.pm

package Disk;
use Exporter ();
@ISA = qw(Exporter);
@EXPORT = qw(setquota);

# We deprecated the usage of perl-Quota and instead use 'repquota' and 'setquota' from
# the RPM "quota". Please note: We need quota v4.0.4 or better, because we use the "-O csv"
# parameter, which isn't present in earlier versions. Special note: CentOS 7 uses an older 
# quota RPM, so we supply our own that is new enough. On CentOS 8 we use the OS supplied one.
$repquota_cmd = '/usr/sbin/repquota';
$setquota_cmd = '/usr/sbin/setquota';

#use vars qw($DEBUG);

# Debugging switch:
$DEBUG = "0";
if ($DEBUG)
{
    use Sys::Syslog qw( :DEFAULT setlogsock);
    use Data::Dumper;
}

use CCE;
use Base::HomeDir;
use DiskInfo;
use Sauce::Util;

sub setquota
# Set the quota for a user or group.
# Arguments: cce connection, user or group object, and oid
# Side effects: modifies the quota tables
# Returns: 1 for success, a string explaining failure otherwise.
{
    my ($cce, $obj, $oid) = @_;

    my $type = $obj->{CLASS};
    my $name = $obj->{name};

    my ($ok, $disk, $disk_old) = $cce->get($oid, 'Disk');
    my $limit = $disk->{quota};

    # be explicit about this on destroy to avoid confusion in the future
    if ($cce->event_is_destroy()) {
        # always set to unlimited on destroy
        $limit = -1;
    }
    
    my @dirs = ();
    my $home = $Base::HomeDir::HOME_ROOT;
    my $BlocksPerMB = $DiskInfo::BYTES_PER_BLOCK;
    # user or group
    &debug_msg("Creation type is: $type \n");
    &debug_msg("Creation limit is: $limit \n");
    &debug_msg("Creation disk is: $disk \n");
    if ($type eq 'User') {
        $type = 0;
        if (($name eq "admin") || ($name eq "alter-admin")) {
                # If we're creating the 'admin' User, we don't really need quota for him.
                # We need the admin account so much that we simply ignore any quota issues
                # that could potentially arise.
                &debug_msg("Creation name is: $name - returning 1 (success)\n");
                return 1;
        }
        # volume for users lives in main namespace
        push @dirs, ($obj->{volume} ? $obj->{volume} : $home);
    } else {
        $type = 1;
        # volume for Workgroups and Vsites will be in main namespace
        push @dirs, ($obj->{volume} ? $obj->{volume} : $home);
    }

    for my $dir (@dirs) {
        my $softquota = 0;
        my $hardquota = 0;
        my $softinode = 0;
        my $hardinode = 0;

        if ($limit eq 0) {
            # no quota is really quota for one file
            $softquota = 1;
            $hardquota = 1;
            $softinode = 1;
            $hardinode = 1;
        }
        elsif ($limit gt 0) {
            $softquota = $limit * $BlocksPerMB;
            $hardquota = $softquota + $BlocksPerMB;
        } 

        # Get existing quota:
        my @args = (Disk::getquota($cce, $obj, $oid));

        # harris used extra arg $type here, which is not documented
        # to let it set quotas for groups too --pbaltz
        if ($type) {
            # use 0 (defaults to this anyway), "" still gives error
            push @args, 0; 
            push @args, $type;
        }

        # Set Quota using 'setquota':
        # setquota [ -rm ] [ -u | -g | -P ] [ -F quotaformat ] name block-softlimit block-hardlimit inode-softlimit inode-hardlimit -a | filesystem...

        # volume for users lives in main namespace
        $volume = '/home';
        $haveHome = `mount|grep "/home"|wc -l`;
        chomp($haveHome);
        if ($haveHome eq 0) {
            $volume = '/';
        }
        if ($dir) {
            $haveDirQuota = `mount|grep "$dir"|wc -l`;
            chomp($haveDirQuota);
            if ($haveDirQuota eq 1) {
                $volume = $dir;
            }
        }
        &debug_msg("Using Volume: $volume \n");

        if ($type eq 0) {
            # Set Quota for User:
            $quota_command = "LC_ALL=C $setquota_cmd -u $name --always-resolve $softquota $hardquota $softinode $hardinode $volume";
            &debug_msg("Set Quota: \"$quota_command\"\n");
            $exit_code = system($quota_command);
        }
        else {
            # Set Quota for Group:
            $quota_command = "LC_ALL=C $setquota_cmd -g $name --always-resolve $softquota $hardquota $softinode $hardinode $volume";
            &debug_msg("Set Quota: \"$quota_command\"\n");
            $exit_code = system($quota_command);
        }
        &debug_msg("Set Quota ret: $exit_code\n");

        #ROLLBACK QUOTA
        &debug_msg("Quota operation args: " . join(' ', @args) . " \n");
        if ($exit_code != 0) {
            &debug_msg("Quota Error: Exit code $exit_code\n");
            &debug_msg("Quota Error args: $quota_command \n");
            $cce->warn('couldNotSetQuota', {'name' => $name });
            return 0;
        }

        # add rollback for quota
        my $old_softquota = 0;
        my $old_hardquota = 0;
        my $old_softinode = 0;
        my $old_hardinode = 0;
        if ($disk_old->{quota} eq 0) {
            # no quota is really quota for one file
            $old_softquota = 1;
            $old_hardquota = 1;
            $old_softinode = 1;
            $old_hardinode = 1;
        } elsif ($disk_old->{quota} > 0) {
            $old_softquota = $disk_old->{quota} * $BlocksPerMB;
            $old_hardquota = $old_softquota + $BlocksPerMB;
        }

        if ($type eq 0) {
            # Define rollback_cmd for User:
            $rollback_cmd = "LC_ALL=C $setquota_cmd -u $name --always-resolve $old_softquota $old_hardquota $old_softinode $old_hardinode $volume";
            $exit_code = system($quota_command);
        }
        else {
            # Define rollback_cmd Group:
            $rollback_cmd = "LC_ALL=C $setquota_cmd -g $name --always-resolve $old_softquota $old_hardquota $old_softinode $old_hardinode $volume";
            $exit_code = system($quota_command);
        }
        &debug_msg("Rollback: $rollback_cmd \n");
        Sauce::Util::addrollbackcommand($rollback_cmd);
    }
    return 1;
}

sub getquota
# Get the quota for a user or group.
# Arguments: cce connection, user or group object, and oid
# Returns: 1 for success, a string explaining failure otherwise.
{

    my ($cce, $obj, $oid) = @_;

    my $type = $obj->{CLASS};
    my $name = $obj->{name};

    my ($ok, $disk, $disk_old) = $cce->get($oid, 'Disk');
    my $limit = $disk->{quota};

    # be explicit about this on destroy to avoid confusion in the future
    if ($cce->event_is_destroy()) {
        # always set to unlimited on destroy
        $limit = -1;
    }
    
    my @dirs = ();
    my @quotaresult =();
    my @gqres =();
    my $home = $Base::HomeDir::HOME_ROOT;
    my $BlocksPerMB = $DiskInfo::BYTES_PER_BLOCK;
    # user or group
    &debug_msg("Quota type is: $type \n");
    &debug_msg("Quota limit is: $limit \n");
    &debug_msg("Quota disk is: $disk \n");
    &debug_msg("Quota oid is: $oid \n");

    # volume for users lives in main namespace
    $volume = '/home';
    $haveHome = `mount|grep "/home"|wc -l`;
    chomp($haveHome);
    if ($haveHome eq 0) {
        $volume = '/';
    }
    if ($dir) {
        $haveDirQuota = `mount|grep "$dir"|wc -l`;
        chomp($haveDirQuota);
        if ($haveDirQuota eq 1) {
            $volume = $dir;
        }
    }

    if ($type eq 'User') {
        $type = 0;
        if (($name eq "admin") || ($name eq "alter-admin")) {
                # If we're creating the 'admin' User, we don't really need quota for him.
                # We need the admin account so much that we simply ignore any quota issues
                # that could potentially arise.
                &debug_msg("Quota name is: $name - returning 1 (success)\n");
                return 1;
        }
    } 
    else {
        $type = 1;
    }

    # LC_ALL=C /usr/sbin/repquota -u -O csv /
    # User,BlockStatus,FileStatus,BlockUsed,BlockSoftLimit,BlockHardLimit,BlockGrace,FileUsed,FileSoftLimit,FileHardLimit,FileGrace

    if ($type eq 0) {
        # Get Quota for User:
        &debug_msg("Getting User Quota: '$repquota_cmd -u -O csv $volume|grep \"^$name,\"'\n");
        $currQuota = `LC_ALL=C $repquota_cmd -u -O csv $volume|grep "^$name,"`;
    }
    elsif ($type eq 1) {
        # Get Quota for Group:
        &debug_msg("Getting Group Quota: '$repquota_cmd -g -O csv $volume|grep \"^$name,\"'\n");
        $currQuota = `LC_ALL=C $repquota_cmd -g -O csv $volume|grep "^$name,"`;
    }
    else {
        &debug_msg("Not specified if we wanted User or Group Quota! Getting both then: '$repquota_cmd -ug -O csv $volume|grep \"^$name,\"'\n");
        $currQuota = `LC_ALL=C $repquota_cmd -ug -O csv $volume|grep "^$name,"`;
    }
    &debug_msg("getquota result for $name: $currQuota \n");
    @gqres = split /,/, $currQuota;
    @quotaresult = ($volume, $gqres[0], $gqres[4], $gqres[5], $gqres[8], $gqres[9]);
    return @quotaresult;
}

sub getquotausage
# Get the quota *and* quota usage for a user or group.
# Arguments: cce connection, user or group object, and oid
# Returns: 1 for success, a string explaining failure otherwise.
{

    my ($cce, $obj, $oid) = @_;

    my $type = $obj->{CLASS};
    my $name = $obj->{name};

    my ($ok, $disk, $disk_old) = $cce->get($oid, 'Disk');
    my $limit = $disk->{quota};

    # be explicit about this on destroy to avoid confusion in the future
    if ($cce->event_is_destroy()) {
        # always set to unlimited on destroy
        $limit = -1;
    }
    
    my @dirs = ();
    my $quotaresult = {};
    my @gqres =();
    my $home = $Base::HomeDir::HOME_ROOT;
    my $BlocksPerMB = $DiskInfo::BYTES_PER_BLOCK;
    # user or group
    &debug_msg("Quota type is: $type \n");
    &debug_msg("Quota limit is: $limit \n");
    &debug_msg("Quota disk is: $disk \n");
    &debug_msg("Quota oid is: $oid \n");

    # volume for users lives in main namespace
    $volume = '/home';
    $haveHome = `mount|grep "/home"|wc -l`;
    chomp($haveHome);
    if ($haveHome eq 0) {
        $volume = '/';
    }
    if ($dir) {
        $haveDirQuota = `mount|grep "$dir"|wc -l`;
        chomp($haveDirQuota);
        if ($haveDirQuota eq 1) {
            $volume = $dir;
        }
    }

    if ($type eq 'User') {
        $type = 0;
        if (($name eq "admin") || ($name eq "alter-admin")) {
                # If we're creating the 'admin' User, we don't really need quota for him.
                # We need the admin account so much that we simply ignore any quota issues
                # that could potentially arise.
                &debug_msg("Quota name is: $name - returning 1 (success)\n");
                return 1;
        }
    } 
    else {
        $type = 1;
    }

    # LC_ALL=C /usr/sbin/repquota -u -O csv /
    # User,BlockStatus,FileStatus,BlockUsed,BlockSoftLimit,BlockHardLimit,BlockGrace,FileUsed,FileSoftLimit,FileHardLimit,FileGrace

    if ($type eq 0) {
        # Get Quota for User:
        &debug_msg("Getting User Quota: '$repquota_cmd -u -O csv $volume|grep \"^$name,\"'\n");
        $currQuota = `LC_ALL=C $repquota_cmd -u -O csv $volume|grep "^$name,"`;
    }
    elsif ($type eq 1) {
        # Get Quota for Group:
        &debug_msg("Getting Group Quota: '$repquota_cmd -g -O csv $volume|grep \"^$name,\"'\n");
        $currQuota = `LC_ALL=C $repquota_cmd -g -O csv $volume|grep "^$name,"`;
    }
    else {
        &debug_msg("Not specified if we wanted User or Group Quota! Getting both then: '$repquota_cmd -ug -O csv $volume|grep \"^$name,\"'\n");
        $currQuota = `LC_ALL=C $repquota_cmd -ug -O csv $volume|grep "^$name,"`;
    }
    &debug_msg("getquota result for $name: $currQuota \n");
    @gqres = split /,/, $currQuota;
    $quotaresult = { 'BlockUsed' => $gqres[3], 'BlockSoftLimit' => $gqres[4], 'BlockHardLimit' => $gqres[5], 'FileSoftLimit' => $gqres[8], 'FileHardLimit' => $gqres[9]};

    return $quotaresult;
}


sub debug_msg {
    if ($DEBUG) {
        my $msg = shift;
        setlogsock('unix');
        openlog($0,'','user');
        syslog('info', "$ARGV[0]: $msg");
        closelog;
    }
}

1;

# 
# Copyright (c) 2008-2019 Michael Stauber, SOLARSPEED.NET
# Copyright (c) 2008-2019 Team BlueOnyx, BLUEONYX.IT
# Copyright (c) 2003 Sun Microsystems, Inc. 
# All Rights Reserved.
# 
# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its 
#    contributors may be used to endorse or promote products derived 
#    from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# 
# You acknowledge that this software is not designed or intended for 
# use in the design, construction, operation or maintenance of any 
# nuclear facility.
# 
